// 完毕：
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.AcroForms;
using GrapeCity.Documents.Pdf.Graphics;
using GrapeCity.Documents.Text;
using System.Security.Cryptography.X509Certificates;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // 此示例演示了如何使用 .pfx 文件创建 PDF 并对其进行签名，
    // 指定自定义 AppearanceStream 来表示签名。
    // 这个示例与SignDoc类似，只不过它添加了AppearanceStream。
    public class SignatureAppearance
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            var tf = new TextFormat() { Font = StandardFonts.Times, FontSize = 14 };
            page.Graphics.DrawString("你好世界！" +
                "",
                tf, new PointF(72, 72));

            // 初始化测试证书：
            var pfxPath = Path.Combine("Resources", "Misc", "DsPdfTest.pfx");
            var cert = new X509Certificate2(File.ReadAllBytes(pfxPath), "qq",
                X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);
            var sp = new SignatureProperties()
            {
                SignatureBuilder = new Pkcs7SignatureBuilder()
                {
                    CertificateChain = new X509Certificate2[] { cert }
                },
                Location = "DsPdfWeb Demo Browser",
                SignerName = "DsPdfWeb",
                SigningDateTime = Common.Util.TimeNow(),
            };

            // 创建一个签名字段来保存签名：
            var sf = new SignatureField();
            // 将签名字段添加到文档中：
            doc.AcroForm.Fields.Add(sf);
            // 连接签名字段和签名属性：
            sp.SignatureField = sf;

            // 设置签名字段：
            sf.Widget.Rect = new RectangleF(page.Size.Width - 72 * 4, 72 * 2, 72 * 3, 72);
            sf.Widget.Page = page;
            // 小部件视觉道具将被下面设置的 sf.Widget.AppearanceStreams.Normal.Default 覆盖：
            sf.Widget.BackColor = Color.PeachPuff;
            sf.Widget.Border = new GrapeCity.Documents.Pdf.Annotations.Border()
            {
                Color = Color.SaddleBrown,
                Width = 3,
            };
            sf.Widget.ButtonAppearance.Caption = $"Signer: {sp.SignerName}\r\nLocation: {sp.Location}";
            // 创建自定义签名外观流：
            var rc = new RectangleF(PointF.Empty, sf.Widget.Rect.Size);
            var fxo = new FormXObject(doc, rc);
            rc.Inflate(-4, -4);
            fxo.Graphics.FillEllipse(rc, Color.CornflowerBlue);
            fxo.Graphics.DrawEllipse(rc, new GCDRAW.Pen(Color.RoyalBlue, 3));
            rc.Inflate(-5, -5);
            fxo.Graphics.DrawEllipse(rc, new GCDRAW.Pen(Color.LightSteelBlue, 1));
            fxo.Graphics.DrawString($"Signed by {sp.SignerName}\non {Common.Util.TimeNow():yyyy-MM-dd}.",
                new TextFormat()
                {
                    FontName = "Times New Roman",
                    FontSize = 14,
                    FontItalic = true,
                    ForeColor = Color.Navy
                },
                fxo.Bounds,
                TextAlignment.Center, ParagraphAlignment.Center, false);
            sf.Widget.AppearanceStreams.Normal.Default = fxo;

            // 重置签名外观以便使用小部件外观流：
            sp.SignatureAppearance = null;

            // 签署并保存文档：
            // 笔记：
            // - 签名和保存是一个原子操作，两者不能分开。
            // - 传递给 Sign() 方法的流必须可读。
            doc.Sign(sp, stream);

            // 完成（生成并签名的文档已保存到“流”）。
            return doc.Pages.Count;
        }
    }
}
